#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <ctype.h>
#include "tetX.h"

extern int     screenNumber;


extern Display	*display;
extern Window	window;
extern Colormap   colmap;
extern GC  graphContext;
extern Visual  *visual;
extern Pixmap  screenContext;
extern int ScreenWidth, ScreenHeight, ScreenDepth; 
extern int BkgrColour;
extern int ConfineMode, Confined;
extern Atom WMProtocolsAtom, WMDeleteWndAtom, WMStateAtom; 
extern char *PositionText;

extern unsigned long PixValues[];
Window root_window;

#define STRT_WIDTH   600
#define STRT_HEIGHT  440

static char * WndName = "Intelligent TETRIS";
static char *IconName = "ITETRIS";

  

int prepare_video(char *DispName)  //, int argc, char **argv)

{   XSetWindowAttributes    attributes;
    unsigned long       attribute_mask;

    XTextProperty	w_name;
    XTextProperty	i_name;

    XSizeHints		sizehints;
    XWMHints		wmhints;
    XClassHint  	classhints;
    int 		StartX, StartY;	
    int			Mask;
    short		PositionSpecified;
    unsigned int        Temp1, Temp2;



    display = XOpenDisplay(DispName);
    BkgrColour = 0;    
    
    if (display == NULL)
    { if (DispName == NULL)
           fprintf(stderr, "This application works under X.\n");
      else
           fprintf(stderr, "Error opening display %s\n", DispName);
      return 1;
    }

    screenNumber = DefaultScreen(display);

    root_window =  RootWindow(display, screenNumber);

    visual = DefaultVisual(display, screenNumber);  // CopyFromParent;    
//    colmap = DefaultColormap(display, screenNumber);

    colmap = XCreateColormap(display, root_window, visual, AllocNone);
    if (colmap == None)
    { fprintf(stderr, "Enable to allocate colourmap.\n");
      return 1;
    }


    /*
     * Set up global attributes
     */         
    ScreenWidth = STRT_WIDTH;
    ScreenHeight = STRT_HEIGHT;
    ScreenDepth = DefaultDepth(display, screenNumber);


    /*
     * Initializing colourmap
     */ 
    initPalette();


    /*
     * Set up window attributes
     */
    attributes.background_pixel = PixValues[0];  // BkgrColour];
    attributes.border_pixel     = BlackPixel(display, screenNumber);
    attributes.event_mask       = ExposureMask | KeyPressMask;
				/*  StructureNotifyMask | LeaveWindowMask; */
    if (ConfineMode)  attributes.event_mask |= ButtonPressMask;

/*   attributes.bit_gravity      = StaticGravity; */
    attributes.colormap		= colmap;
/*  attributes.override_redirect = True;    */
    attribute_mask              = CWBackPixel | CWBorderPixel
                                | CWEventMask | CWColormap; 
                                    /*| CWBitGravity | OverrideRedirect; */

    /* Starting position */
    StartX = StartY = -1;
    PositionSpecified = 0;
    if (PositionText)  
    {  Mask = XParseGeometry(PositionText,
                             &StartX, &StartY, &Temp1, &Temp2);
       if ((Mask & (XValue | YValue)) == (XValue | YValue))
       {      PositionSpecified = 1;
              if(Mask & XNegative) StartX += DisplayWidth(display, screenNumber) -
	                                     STRT_WIDTH-8;
	      if(Mask & YNegative) StartY += DisplayHeight(display, screenNumber) -
	                                     STRT_HEIGHT-25;
       }	       
    }
    
    /*
     * Create the window
     */
    
    window = XCreateWindow( display,
                root_window,
                StartX, StartY,         /* -- location */  
                 STRT_WIDTH, STRT_HEIGHT,    /* -- size */
                1,			/* -- Border width */ 
                CopyFromParent,         /* -- Depth */
                InputOutput,            /* -- window class */
                visual,
                attribute_mask,
                &attributes );

    if (window == None) 
    { fprintf (stderr, "Error creating Window\n");
      return 1;
    }  
        

    /*
     * Generate window and icon names.
    */
     XStringListToTextProperty(&WndName,
 			1,	/* 1 string to convert */
			&w_name);

     XStringListToTextProperty(&IconName,
			1,	/* 1 string to convert */
			&i_name);


    /*
     * Fill in size hints
    */ 

    sizehints.base_height = STRT_HEIGHT;
    sizehints.base_height = STRT_HEIGHT;

    sizehints.min_width  = STRT_WIDTH;   
    sizehints.min_height = STRT_HEIGHT;  

    sizehints.max_width  = STRT_WIDTH;   
    sizehints.max_height = STRT_HEIGHT;  

    sizehints.flags =   PBaseSize | PMinSize | PMaxSize; /* | PWinGravity; */

    if (PositionSpecified)
    {
         sizehints.flags |= USPosition;
         sizehints.x = StartX; 
         sizehints.y = StartY;
    }

     
    /* 
     * Fill in "WM" hints.
     */ 
    wmhints.initial_state = NormalState;       /* I.e non-iconic */
    wmhints.input         = True;              /* Keyboard from window is needed */
    wmhints.flags         = StateHint | InputHint;

   
    /*
     * Fill in class name.
     */
    classhints.res_name  = "XITETRIS";
    classhints.res_class = "ACE_XAPPS"; 
    
    XSetWMProperties(display, window,
        &w_name,
        &i_name,
	0, 0, 		/*  argv, argc, Note reversed order. */
        &sizehints,
        &wmhints,
        &classhints);

    XFree(w_name.value);
    XFree(i_name.value);


    /*
     *  Configure window to handle WM_DELETE_WINDOW Event
     */
    WMProtocolsAtom = XInternAtom(display, "WM_PROTOCOLS", True);
    WMDeleteWndAtom = XInternAtom(display, "WM_DELETE_WINDOW", True); 
    WMStateAtom = XInternAtom(display, "WM_STATE", True); 

    if (WMProtocolsAtom != None && WMDeleteWndAtom != None)
           XSetWMProtocols(display, window, &WMDeleteWndAtom, 1);

   
    /*
     * Greate a grahic context.
     */


//    xgcv.graphics_exposures = True;    
    graphContext = XCreateGC( display, window, 0, 0);
    if (graphContext == None)
    { fprintf(stderr, "Error creating GC.\n");
      return 1;
    }

    
    screenContext =  XCreatePixmap(display, window,
                                    sizehints.max_width, 
				    sizehints.max_height,
				    ScreenDepth);
   if (screenContext == None)
    { fprintf(stderr, "Error creating screen context.\n");
      return 1;
    }

    
    if (InitPixmapPats()) return 1;

  { int mask = KeyPressMask | ExposureMask;  // | VisibilityChangeMask;
	if (ConfineMode) mask |= ButtonPressMask;	  

	XSelectInput(display, window, mask);
  }	

    
    return 0;
}


void terminate_video(void)
{  

    FreePixmapPats();
    
    if (display != NULL)
    {   deinitPalette();
       if (graphContext != None) XFreeGC(display, graphContext);
       if (screenContext != None) XFreePixmap(display, screenContext);
       if (colmap != None) XFreeColormap(display, colmap);
       if (window != None) XDestroyWindow(display, window);
       XCloseDisplay(display);	
    }        
	      
}
